/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2007 Smithsonian Astrophysical Observatory
 *
 * This file is part of enwtsum
 *
 * enwtsum is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>

#include <rbtree/rbtree.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

#include "main.h"

/* Routine to get the data pointer for a named data packet field */

static char *
dpktf_data( BPipe *bpipe, char *name, void *data )
{
  DpktField *field = bpipe_dpktf( bpipe, name );

  if ( field == NULL )
    tf_exit( ExitERR_misc, "%s isn't in input data stream", name);

  return bpipe_dpktf_data( field, data );

}

void
process(Common *c, RBTree tree)
{
  BPipe *bpipe;
  BPipeOutput *bpo;

  void *data;
  double *energy, *weight;

  size_t dpkt_core_size;
  size_t nphot;

  tf_enter("process");

/* Setup bpipe input and output streams */

  if ( ( bpipe = bpipe_new() ) == NULL)
    tf_exit(ExitERR_misc, "couldn't open bpipe: %s",
	    bpipe_strerror(bpipe_errno));

  if ( ( bpipe_input(bpipe, c->input) ) != NULL)
    tf_exit(ExitERR_misc, "couldn't open input file `%s': %s", c->input,
	    bpipe_strerror(bpipe_errno));

  if ( NULL == ( bpo = bpipe_output( bpipe, c->output ) ) )
    tf_exit(ExitERR_misc, "couldn't open input file `%s': %s", c->output,
	    bpipe_strerror(bpipe_errno));

  if (NULL == (data = bpipe_map_alloc(bpipe,1, &dpkt_core_size)))
    tf_exit( ExitERR_alloc, "unable to allocate memory for data packet: %s",
	    bpipe_strerror(bpipe_errno));

  energy = (double *) dpktf_data( bpipe, BP_AXAF_phot_energy, data);
  weight = (double *) dpktf_data( bpipe, BP_AXAF_phot_wt, data);

  if ( bpipe_write_hdr(bpipe) )
    tf_exit(ExitERR_misc,"bpipe_write_hdr: %s",bpipe_strerror(bpipe_errno));
  
  nphot = 0;
  bpipe_errno = BPNOERROR;

  while( bpipe_read_dpkts( bpipe, data, 1 )   == 1) 
  {
    EnWtSum *ews;
    nphot++;

    /* check if this energy is already in the tree */
    if ( NULL ==
	 ( ews = (EnWtSum *) rbtree_search( tree, energy, key_cmp ) )
	 )
    {
      if ( NULL == ( ews = (EnWtSum *) malloc( sizeof(EnWtSum) ) ) )
	tf_exit( ExitERR_alloc, "unable to allocate struct for data\n");

      ews->energy = *energy;
      ews->wt = *weight;
      ews->n = 1;
      rbtree_insert( tree, ews );
    }

    /* it's in the tree, just update the weight and count */
    else
    {
      ews->wt += *weight;
      ews->n++;
    }

    /* write the ray out */
    if (bpipe_write_dpkt( bpipe, data, bpo))
       tf_exit( ExitERR_misc, "error writing photon # %d: %s", nphot,
		bpipe_strerror(bpipe_errno) ); 
  }

  if ( bpipe_errno != BPNOERROR )
    tf_exit( ExitERR_misc, "error reading photon %d: %s", nphot + 1,
	     bpipe_strerror( bpipe_errno) );

  bpipe_delete( bpipe );

  tf_leave();
}

